from __future__ import division
from math import *
from SDL import *
from OpenGL.GL import *
from helpers import *
from game import *

class SimpleDraw:
    def __init__(self):
        self.resize()
        glEnable(GL_BLEND)
        glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)

        sf = 1 / 320
        self.font = Font("DejaVuSans.ttf", 12, sf)
        pics = Image.from_ini("cropped.ini", sf)
        self.drawers = {}
        self.pics = {}
        for n, num in sorted(pics):
            self.pics[n] = self.pics.get(n, [])
            self.pics[n].append(pics[n, num])

        def draw_building(pics):
            def draw(o):
                f = 15 * o.timer // o.build_time
                x, y = o.pos
                pics[f].draw(x, y / 2)
            return draw

        def draw_imp(pics):
            def draw(o):
                df = int(32 - (o.dir * 16 + 1 / 32) / 2 / pi) % 16
                mf = (4 + o.movement) % 16
                x, y = o.pos
                pics[mf + 16 * df].draw(x, y / 2)
            return draw
                
        self.drawers["BuilderBush"] = draw_building(self.pics["bush"])
        self.drawers["PlanterBush"] = draw_building(self.pics["flowerbush"])
        self.drawers["Collector"] = draw_imp(self.pics["imp"])
        self.drawers["Builder"] = draw_imp(self.pics["imp"])
        self.drawers["Planter"] = draw_imp(self.pics["imp"])
        self.drawers["Flower"] = draw_building(self.pics["flower"])
        self.drawers["Cave"] = draw_building(self.pics["cave"])
        self.drawers["Tree"] = draw_building(self.pics["flowertree"])
        self.drawers["Elf"] = draw_imp(self.pics["elf"])
        self.drawers["Caterp"] = draw_imp(self.pics["caterp"])
        self.drawers["DefenseTree"] = draw_building(self.pics["tree"])
        self.drawers["DefenderElf"] = draw_imp(self.pics["elf"])
        self.drawers["HealerElf"] = draw_imp(self.pics["elf"])
        self.drawers["HealerBush"] = draw_building(self.pics["healerbush"])

        self.lawnpic = self.pics["lawn"][0]

    def resize(self):
        s = SDL_GetVideoSurface()
        glViewport(0, 0, s.w, s.h);
        glMatrixMode(GL_PROJECTION)
        glLoadIdentity()
        a = s.h / s.w
        glOrtho(-1, 1, -a, a, -1, 1)
        glMatrixMode(GL_MODELVIEW)

    def circle(self, x, y, r):
        n = max(3, int(2 * r * pi / 0.03))
        self.v = [(x + r * cos(i * 2 * pi / n),
            y + r * sin(i * 2 * pi / n) * 0.5) for i in range(n)]
        glVertexPointerf(self.v)
        glDrawArrays(GL_LINE_LOOP, 0, n)

    def draw_game(self, game):
        glClearColor(0.6, 0.8, 1, 1)
        glClear(GL_COLOR_BUFFER_BIT)
        glEnableClientState(GL_VERTEX_ARRAY)
        glEnable(GL_TEXTURE_2D)
        glEnableClientState(GL_TEXTURE_COORD_ARRAY)

        x, y = game.pos
        if self.lawnpic:
            self.lawnpic.draw(x, y)
        else:
            self.circle(x, y, game.radius)

        glDisableClientState(GL_TEXTURE_COORD_ARRAY)
        glDisable(GL_TEXTURE_2D)

        drawlist = sorted(game.objects, key = lambda x:
            x.layer * 1000 - x.pos[1])

        for o in drawlist:
            name = o.__class__.__name__
            if not name in self.drawers:
                x, y = o.pos
                self.circle(x, y / 2, o.radius)

        for o in game.buildings:
            if o.timer < o.build_time:
                x, y = o.pos
                self.circle(x, y / 2, o.radius)

        glEnable(GL_TEXTURE_2D)
        glEnableClientState(GL_TEXTURE_COORD_ARRAY)
        glColor3f(1, 1, 1)

        for o in drawlist:
            name = o.__class__.__name__
            if name in self.drawers:
                self.drawers[name](o)
            else:
                x, y = o.pos
                self.font.draw(name, x, (y + o.radius + 0.05) / 2,
                    flags = Font.Center)
            
        self.font.draw("Lawn", 0, 0.6, flags = Font.Center)

        l = self.font.line_height
        y = 1.25 / 2
        self.font.draw("Upkeep", 0.5, y)
        self.font.draw("%d/%d" % (game.upkeep, game.max_upkeep), 0.85, y)
        y -= l
        self.font.draw("Nectar", 0.5, y)
        self.font.draw("%d" % game.nectar, 0.85, y)
        y -= l
        self.font.draw("Time", 0.5, y)
        secs = game.ticks // 30
        self.font.draw("%02d:%02d" % (secs // 60, secs % 60), 0.85, y)
        y -= l
        self.font.draw("Get off my lawn", 0.5, y)
        self.font.draw("%d/%d" % (game.enum, game.max_enum), 0.85, y)

        x = 0.7
        y = -1
        if game.cave and game.cave.removed:
            self.font.draw("you fail", x, y / 2)
        if game.enum == game.max_enum and len(game.enemies) == 0:
            self.font.draw("you win", x, y / 2)

        glDisableClientState(GL_TEXTURE_COORD_ARRAY)
        glDisable(GL_TEXTURE_2D)

        glDisableClientState(GL_VERTEX_ARRAY)

    def draw_ui(self, ui, game, mpos):
        glEnableClientState(GL_VERTEX_ARRAY)
        glColor3f(1, 1, 1)

        mouseover = True

        if game.can_build(mpos):
            x, y = mpos
            self.circle(x, y / 2, game.build.radius)
            mouseover = False
        
        for b in ui.buttons:
            x, y = b.pos
            self.circle(x, y / 2, b.radius)

        glEnable(GL_TEXTURE_2D)
        glEnableClientState(GL_TEXTURE_COORD_ARRAY)

        if mouseover:
            for o in game.board.get_objects(mpos, 0.01):
                if isinstance(o, (Imp, Caterp)):
                    x, y = o.pos
                    self.font.draw(o.state, x, y / 2 - self.font.line_height)
                    self.font.draw("%d/%d hp" % (o.hp, o.maxhp), x, y / 2)
                    mouseover = False
                    break
                elif isinstance(o, (Building)):
                    x, y = o.pos
                    self.font.draw("%d/%d hp" % (o.hp, o.maxhp), x, y / 2 )
                    mouseover = False
                    break

        for b in ui.buttons:
            x, y = b.pos
            self.font.draw(b.name, x, y / 2, flags = Font.Center)

        b = ui.over(mpos)
        if b:
            x, y = mpos
            x += 0.05
            y -= 0.05
            if y - self.font.line_height * 2 * len(b.tooltip) < -1.25:
                y = -1.25 + self.font.line_height * 2 * len(b.tooltip)
            for line in b.tooltip:
                self.font.draw(line, x, y / 2)
                y -= self.font.line_height * 2

        glDisableClientState(GL_TEXTURE_COORD_ARRAY)
        glDisable(GL_TEXTURE_2D)
        glDisableClientState(GL_VERTEX_ARRAY)
